/*****************************************************************************
 * © 2014 Avaya Inc. All rights reserved.
 ****************************************************************************/
package com.avaya.services.Callingpolicies;

import com.avaya.collaboration.call.Call;
import com.avaya.collaboration.call.Participant;
import com.avaya.collaboration.call.media.MediaFactory;
import com.avaya.collaboration.call.media.PlayItem;
import com.avaya.collaboration.call.media.DigitOptions;
import com.avaya.collaboration.call.media.MediaService;
import com.avaya.collaboration.util.logger.Logger;

public final class PromptAndCollectOperationImpl implements PromptAndCollectOperation
{
    private final ClusterAttributeReader clusterAttributeReader;
    private final TrafficInterfaceAddressRetriever trafficInterfaceAddressRetriever;

    private final Logger logger;

    public PromptAndCollectOperationImpl()
    {
        this(new ClusterAttributeReaderImpl(), new TrafficInterfaceAddressRetrieverImpl(), Logger
                .getLogger(PromptAndCollectOperationImpl.class));
    }

    PromptAndCollectOperationImpl(final ClusterAttributeReader clusterAttributeReader,
            final TrafficInterfaceAddressRetriever trafficInterfaceAddressRetriever, final Logger logger)
    {
        this.clusterAttributeReader = clusterAttributeReader;
        this.trafficInterfaceAddressRetriever = trafficInterfaceAddressRetriever;
        this.logger = logger;
    }

    /**
     * Prompt the caller to enter a digit to determine the call behaviour.
     * First, we will use DigitOptions to indicate that we want to collect 1
     * digit. Then, we will use PlayItem to specify the announcement we will use
     * for the prompt.
     */
    @Override
    public void executePromptAndCollectOperation(final Call call)
    {
        call.enableMediaBeforeAnswer();

        final DigitOptions digitOptions = MediaFactory.createDigitOptions();
        digitOptions.setNumberOfDigits(1);
        digitOptions.setTerminationKey("#");
        digitOptions.setTimeout(DigitOptions.DEFAULT_TIMEOUT_MILLISECONDS);

        final PlayItem playItem = MediaFactory.createPlayItem();
        playItem.setInterruptible(true);
        playItem.setIterateCount(1);

        try
        {
            final String announcementFileNetworkProtocol =
                    clusterAttributeReader.getAnnouncementFileNetworkProtocol();
            final String trafficInterfaceAddress =
                    trafficInterfaceAddressRetriever.getTrafficInterfaceAddress();
            final String source =
                    announcementFileNetworkProtocol + "://" + trafficInterfaceAddress +
                            "/services/Callingpolicies/Callingpolicies.wav";
            playItem.setSource(source);
        }
        catch (final Exception e)
        {
            logger.warn("executePromptAndCollectOperation: exception=" + e);
            call.drop();
        }

        if (logger.isFinestEnabled())
        {
            logger.finest("executePromptAndCollectOperation: call=" + call + "; prompt and collect=" +
                    playItem + "; digit options=" + digitOptions);
        }

        final MediaService mediaService = MediaFactory.createMediaService();
        final Participant participant = call.getCallingParty();
        final CallingPoliciesMediaListener callingPoliciesMediaListener =
                new CallingPoliciesMediaListener(call);
        mediaService.promptAndCollect(participant, playItem, digitOptions, callingPoliciesMediaListener);
    }
}
